@@ -14,10 +14,12 @@ point but have tested the other one as well --- see nist directory.)
 
 * The results from the fit are compared (see assert statements) with NIST's
 "certified values". All work with the gsl_multifit and scipy_least_squares
-fitters, at least on my Mac.
+fitters, at least on my Mac. (The default GSL fitter is slightly off for 
+lanczos1 but the difference is less than sigma/10; not an issue for the others,
+and not really an issue for the default fitter.)
 
 """
-# Copyright (c) 2016-20 G. Peter Lepage.
+# Copyright (c) 2016-23 G. Peter Lepage.
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -86,9 +88,16 @@ def assert_equal(x, xstr):
         assert ''.join(str(x.tolist()).split(',')) == xstr
     except AssertionError as e:
         ex = e
-        print('not equal:')
-        print('    ', ''.join(str(x.tolist()).split(',')))
-        print('    ', xstr)
+        y = gv.gvar(xstr[1:-1].split())
+        for diff in (x - y):
+            if abs(diff.mean) > diff.sdev / 10:
+                print('not equal:')
+                print('    ', ''.join(str(x.tolist()).split(',')))
+                print('    ', xstr)
+                break
+        else:
+            print('agree to sigma/10')
+
 
 def misra1a():
     print(20 * '=', 'misra1a')
@@ -753,11 +762,32 @@ def lanczos1():
     x = np.array([ 0.  ,  0.05,  0.1 ,  0.15,  0.2 ,  0.25,  0.3 ,  0.35,  0.4 ,
         0.45,  0.5 ,  0.55,  0.6 ,  0.65,  0.7 ,  0.75,  0.8 ,  0.85,
         0.9 ,  0.95,  1.  ,  1.05,  1.1 ,  1.15])
-    y = np.array([ 2.5134    ,  2.04433337,  1.66840444,  1.36641802,  1.12323249,
-        0.92688972,  0.76793386,  0.63887755,  0.53378353,  0.44793636,
-        0.37758479,  0.31973932,  0.27201308,  0.23249655,  0.19965895,
-        0.17227041,  0.14934057,  0.13007002,  0.11381193,  0.10004156,
-        0.08833209,  0.07833544,  0.06976694,  0.06239313])
+    y = np.array([
+        2.513400000000e+00,
+        2.044333373291e+00,
+        1.668404436564e+00,
+        1.366418021208e+00,
+        1.123232487372e+00,
+        9.268897180037e-01,
+        7.679338563728e-01,
+        6.388775523106e-01,
+        5.337835317402e-01,
+        4.479363617347e-01,
+        3.775847884350e-01,
+        3.197393199326e-01,
+        2.720130773746e-01,
+        2.324965529032e-01,
+        1.996589546065e-01,
+        1.722704126914e-01,
+        1.493405660168e-01,
+        1.300700206922e-01,
+        1.138119324644e-01,
+        1.000415587559e-01,
+        8.833209084540e-02,
+        7.833544019350e-02,
+        6.976693743449e-02,
+        6.239312536719e-02,
+        ])
     y = y + gv.gvar(len(y) * ['0 +- 8.9156129349E-14'])
     def fcn(x, b):
         x = x
@@ -772,7 +802,8 @@ def lanczos1():
     print(fit)
     # print(fit.p)
     # assert_equal(fit.p, '[0.0950994 +- 5.3e-11 0.999997 +- 2.7e-10 0.8607 +- 1.4e-10 3 +- 3.3e-10 1.5576 +- 1.9e-10 5 +- 1.1e-10]')
-    assert_equal(fit.p, '[0.095099425639(53) 0.99999658432(27) 0.86069977200(14) 2.99999778670(33) 1.55760080133(19) 4.99999965560(11)]')
+    # assert_equal(fit.p, '[0.095099425639(53) 0.99999658432(27) 0.86069977200(14) 2.99999778670(33) 1.55760080133(19) 4.99999965560(11)]')
+    assert_equal(fit.p, '[0.095100000027(53) 1.00000000013(27) 0.86070000013(14) 3.00000000023(33) 1.55759999984(19) 5.00000000011(11)]')
 
 def lanczos2():
     print(20 * '=', 'lanczos2')
