@@ -4,7 +4,7 @@
 test-lsqfit.py
 
 """
-# Copyright (c) 2012-2018 G. Peter Lepage.
+# Copyright (c) 2012-2021 G. Peter Lepage.
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -267,11 +267,11 @@ class test_lsqfit(unittest.TestCase,ArrayTests):
             data=y, prior=prior, fcn=f, svdcut=1e-15, tol=tol,
             )
         out = [
-            'Least Square Fit:',
+            'Least Squares Fit:',
             '  chi2/dof [dof] = 0.3 [2]    Q = 0.74    logGBF = -2.9682',
             '',
             'Parameters:',
-            '              p    0.90 (44)     [  0.0 (2.0) ]  ',
+            '              p    0.90 (44)     [    0 ± 2.0 ]  ',
             '',
             'Fit:',
             '      key       y[key]    f(p)[key]',
@@ -285,11 +285,11 @@ class test_lsqfit(unittest.TestCase,ArrayTests):
         self.assertEqual(out, fit.format(True).split('\n')[:len(out)])
         self.assertEqual(out, fit.format(True, pstyle='v').split('\n')[:len(out)])
         out = [
-            'Least Square Fit:',
+            'Least Squares Fit:',
             '  chi2/dof [dof] = 0.3 [2]    Q = 0.74    logGBF = -2.9682',
             '',
             'Parameters:',
-            '              p   0.895238 +- 0.436436          [     0 +- 2 ]  ',
+            '              p   0.9 ± 0.44     [    0 ± 2.0 ]  ',
             '',
             'Settings:',
             '  svdcut/n = 1e-15/0    tol = (1e-15,1e-15,1e-15)    (itns/time = 2*/0.0)',
@@ -300,11 +300,11 @@ class test_lsqfit(unittest.TestCase,ArrayTests):
             data=y, prior=prior, fcn=f, svdcut=1e-15, tol=tol,
             )
         out = [
-            'Least Square Fit:',
+            'Least Squares Fit:',
             '  chi2/dof [dof] = 0.3 [2]    Q = 0.74    logGBF = -2.9682',
             '',
             'Parameters:',
-            '              p    0.90 (44)     [  0.0 (2.0) ]  ',
+            '              p    0.90 (44)     [    0 ± 2.0 ]  ',
             '',
             'Settings:',
             '  svdcut/n = 1e-15/0    tol = (1e-15,1e-15,1e-15)    (itns/time = 2*/0.0)',
@@ -324,11 +324,11 @@ class test_lsqfit(unittest.TestCase,ArrayTests):
             p0=p0, data=(x,y), fcn=f, svdcut=None, tol=tol,
             )
         out = [
-            'Least Square Fit (no prior):',
-            '  chi2/dof [dof] = 0.8 [1]    Q = 0.37    ',
+            'Least Squares Fit (no prior):',
+            '  chi2/dof [dof] = 0.8 [1]    Q = 0.37',
             '',
             'Parameters:',
-            '              p    1.09 (19)     [   0 +- inf ]  ',
+            '              p    1.09 (19)     [    0 ± inf ]  ',
             '',
             'Fit:',
             '     x[k]         y[k]    f(x[k],p)',
@@ -350,7 +350,7 @@ class test_lsqfit(unittest.TestCase,ArrayTests):
             prior=prior, data=y, fcn=fcn, debug=True, tol=tol,
             )
         out = [
-            'Least Square Fit:',
+            'Least Squares Fit:',
             '  chi2/dof [dof] = 1.7 [4]    Q = 0.14    logGBF = -2.3346',
             '',
             'Parameters:',
@@ -377,12 +377,12 @@ class test_lsqfit(unittest.TestCase,ArrayTests):
             )
         # print(fit.format(extend=True))
         out = [
-            'Least Square Fit:',
+            'Least Squares Fit:',
             '  chi2/dof [dof] = 1.1 [4]    Q = 0.36    logGBF = -1.77',
             '',
             'Parameters:',
-            '       log(a) 0   -0.020 (90)     [  0.00 (50) ]  ',
-            '              1    0.739 (34)     [  0.00 (50) ]  *',
+            '       log(a) 0   -0.020 (90)     [   0 ± 0.50 ]  ',
+            '              1    0.739 (34)     [   0 ± 0.50 ]  *',
             '------------------------------------------------',
             '            a 0    0.981 (88)     [  1.00 (50) ]  ',
             '              1    2.093 (71)     [  1.00 (50) ]  **',
@@ -412,20 +412,46 @@ class test_lsqfit(unittest.TestCase,ArrayTests):
         fit = nonlinear_fit(data=y, p0=gv.mean(prior), fcn=f, maxit=0)
         np.testing.assert_allclose(gv.mean(fit.p.buf), gv.mean(prior.buf))
 
-    def test_evalchi2(self):
-        y = gv.BufferDict([('a',gv.gvar(1.5,1.0)), ('b',gv.gvar(0.8,0.5))])
-        prior = gv.BufferDict(p=gv.gvar(0,2))
+    def test_pdf_dchi2(self):
+        " fit.pdf(p)  fit.dchi2(p) "
+        y = gv.BufferDict([('a',gv.gvar(1.5,1.0)), ('b',gv.gvar(0.8,0.5)), ('c',gv.gvar(12.,13.))])
+        prior = gv.BufferDict(p=gv.gvar(0,2), q=gvar(0,5))
         def f(p):
-            return dict(a=p['p'], b=p['p'])
+            return dict(a=p['p'], b=p['p'], c=p['p'])
         fit = nonlinear_fit(data=y, prior=prior, fcn=f)
-        self.assertAlmostEqual(fit.evalchi2(fit.pmean), fit.chi2)
-        
-        y = gv.BufferDict([('a',gv.gvar(1.5,1.0)), ('b',gv.gvar(0.8,0.5))])
-        prior = [gv.gvar(0,2)]
+        self.assertAlmostEqual(fit.pdf(fit.pmean), 1.)
+        self.assertAlmostEqual(fit.dchi2(fit.pmean), 0.)
+        p = fit.pmean + fit.psdev 
+        self.assertAlmostEqual(fit.dchi2(p), fit.prior.size)
+        self.assertAlmostEqual(fit.pdf(p), np.exp(-fit.dchi2(p)/2))
+        p['p'] = fit.p['p'].mean
+        self.assertAlmostEqual(fit.dchi2(p), fit.prior.size - 1)
+        self.assertAlmostEqual(fit.pdf(p), np.exp(-fit.dchi2(p)/2))
+
+        y = gv.BufferDict([('a',gv.gvar(1.5,1.0)), ('b',gv.gvar(0.8,0.5)), ('c',gv.gvar(12.,13.))])
+        prior = [gv.gvar(0,2), gv.gvar(0,5)]
         def f(p):
-            return dict(a=p[0], b=p[0])
+            return dict(a=p[0], b=p[0], c=p[0])
         fit = nonlinear_fit(data=y, prior=prior, fcn=f)
-        self.assertAlmostEqual(fit.evalchi2(fit.pmean), fit.chi2)
+        self.assertAlmostEqual(fit.pdf(fit.pmean), 1.)
+        self.assertAlmostEqual(fit.dchi2(fit.pmean), 0.)
+        p = fit.pmean + fit.psdev 
+        self.assertAlmostEqual(fit.dchi2(p), fit.prior.size)
+        self.assertAlmostEqual(fit.pdf(p), np.exp(-fit.dchi2(p)/2))
+        p[0] = fit.p[0].mean
+        self.assertAlmostEqual(fit.dchi2(p), fit.prior.size - 1)
+        self.assertAlmostEqual(fit.pdf(p), np.exp(-fit.dchi2(p)/2))
+
+        y = gv.gvar([gv.gvar(1.5,1.0), gv.gvar(0.8,0.5), gv.gvar(12.,13.)])
+        prior = gv.gvar(0,2)
+        def f(p):
+            return 3 * [p]
+        fit = nonlinear_fit(data=y, prior=prior, fcn=f)
+        self.assertAlmostEqual(fit.pdf(fit.pmean), 1.)
+        self.assertAlmostEqual(fit.dchi2(fit.pmean), 0.)
+        p = fit.pmean + fit.psdev 
+        self.assertAlmostEqual(fit.dchi2(p), 1)
+        self.assertAlmostEqual(fit.pdf(p), np.exp(-fit.dchi2(p)/2))
 
     def test_unusual_cases(self):
         """ unusual cases """
@@ -472,6 +498,7 @@ class test_lsqfit(unittest.TestCase,ArrayTests):
             gv.fmt_errorbudget(outputs=dict(a=fit.p['a']), inputs=dict(prior=fit.prior, data=fit.data)),
             gv.fmt_errorbudget(outputs=dict(a=dfit.p['a']), inputs=dict(prior=dfit.prior, data=dfit.data)),
             )
+
         # redo with lambda function -- can't be pickled
         fit = nonlinear_fit(prior=prior, data=data, fcn=lambda p: 10 * [p['a']], debug=True)
         with warnings.catch_warnings(record=True) as w:
@@ -614,7 +641,7 @@ class test_lsqfit(unittest.TestCase,ArrayTests):
             a = gv.gvar('1(1)')
             b = gv.gvar('2(1)')
             c = gv.gvar('3(1)')
-            avg = wavg([a, b, c], fast=fast)
+            avg = wavg([a, b, c], fast=fast) 
             np.testing.assert_allclose([avg.mean, avg.sdev], [2., 1/3.**0.5])
             a = gv.gvar(dict(a=[['1(1)', '2(1)']], b=['10(1)', '5(3)'], c='1(1)'))
             b = gv.gvar(dict(a=[['2(1)', '1(1)']], b=['20(1)'], c='2(1)'))
@@ -873,7 +900,23 @@ class test_lsqfit(unittest.TestCase,ArrayTests):
             return dict(data=(x, y), prior=prior, fcn=f)
         z0 = dict(z=np.log(0.7))
         fit,z = empbayes_fit(z0, fitargs, tol=1e-3)
-        # self.assertAlmostEqual(np.exp(z['z']), 0.6012, places=1)
+        self.assertAlmostEqual(np.exp(z['z']), 0.6012, places=1)
+
+        # p0
+        def fcn(p):
+            return 3 * [p**2]
+        data = gv.gvar(['12(5)', '3(4)', '20(9)'])
+        def fitargs1(z):
+            prior = gv.gvar('10 +- {}'.format(0.01 + z**2))
+            return dict(data=data, prior=prior, fcn=fcn, p0=10)
+        fit1,z1 = empbayes_fit(5., fitargs1, tol=1e-3)
+        def fitargs2(z):
+            prior = gv.gvar('10 +- {}'.format(0.01 + z**2))
+            return dict(data=data, prior=prior, fcn=fcn)
+        fit2,z2 = empbayes_fit(5., fitargs2, p0=10, tol=1e-3)
+        self.assertAlmostEqual(z1, z2)
+        self.assertEqual(str(fit1.p), str(fit2.p))
+        self.assertGreater(fit1.nit, fit2.nit)
 
     def test_unpack_data(self):
         """ lsqfit._unpack_data """
@@ -905,7 +948,7 @@ class test_lsqfit(unittest.TestCase,ArrayTests):
         p = gv.gvar(2,4)
         po = lsqfit._unpack_gvars({'log(p)':gv.log(p)})
         xo = 20
-        x, y, prior, fdata = lsqfit._unpack_data(
+        x, y, prior, yp_pdf = lsqfit._unpack_data(
             data=(xo,yo), prior=po, svdcut=0, eps=None, debug=True,
             uncorrelated_data=False, **noiseargs
             )
@@ -914,104 +957,104 @@ class test_lsqfit(unittest.TestCase,ArrayTests):
         self.assert_gvclose(gv.exp(prior['log(p)']), p)
         self.assert_gvclose(prior['p'], p)
 
-        self.assert_arraysequal(fdata.mean, [1., 10., gv.log(2.)])
-        self.assert_arraysequal(fdata.inv_wgts[0][0], [0,1,2])
-        self.assert_arraysequal(fdata.inv_wgts[0][1], [0.5, 0.25, 0.5])
-        sumsvd = fdata.correction
+        self.assert_arraysequal(yp_pdf.mean, [1., 10., gv.log(2.)])
+        self.assert_arraysequal(yp_pdf.i_invwgts[0][0], [0,1,2])
+        self.assert_arraysequal(yp_pdf.i_invwgts[0][1], [0.5, 0.25, 0.5])
+        sumsvd = sum(yp_pdf.correction)
         self.assertEqual([sumsvd.mean, sumsvd.sdev], [0, 0])
-        test_logdet(fdata, prior, y)
+        test_logdet(yp_pdf, prior, y)
 
         # case 2 - no x
         po = lsqfit._unpack_gvars(dict(p=gv.gvar(2, 4)))
-        x, y, prior, fdata = lsqfit._unpack_data(
+        x, y, prior, yp_pdf = lsqfit._unpack_data(
             data=yo, prior=po, svdcut=0, eps=None, debug=True,
             uncorrelated_data=False, **noiseargs
             )
         self.assertEqual(x,False)
         self.assert_gvclose(y['y'], yo['y'])
         self.assert_gvclose(prior['p'], po['p'])
-        self.assert_arraysequal(fdata.mean, [1., 10., 2.])
-        self.assert_arraysequal(fdata.inv_wgts[0][0], [0,1,2])
-        self.assert_arraysequal(fdata.inv_wgts[0][1], [0.5, 0.25, 0.25])
-        sumsvd = fdata.correction
+        self.assert_arraysequal(yp_pdf.mean, [1., 10., 2.])
+        self.assert_arraysequal(yp_pdf.i_invwgts[0][0], [0,1,2])
+        self.assert_arraysequal(yp_pdf.i_invwgts[0][1], [0.5, 0.25, 0.25])
+        sumsvd = sum(yp_pdf.correction)
         self.assertEqual([sumsvd.mean, sumsvd.sdev], [0, 0])
-        test_logdet(fdata, prior, y)
+        test_logdet(yp_pdf, prior, y)
 
         # case 3 - no prior, x
-        x, y, prior, fdata = lsqfit._unpack_data(
+        x, y, prior, yp_pdf = lsqfit._unpack_data(
             data=yo, prior=None, svdcut=0, eps=None, debug=True,
             uncorrelated_data=False, **noiseargs
             )
         self.assertEqual(x,False)
         self.assert_gvclose(y['y'], yo['y'])
         self.assertEqual(prior, None)
-        self.assert_arraysequal(fdata.mean, [1., 10.])
-        self.assert_arraysequal(fdata.inv_wgts[0][0], [0,1])
-        self.assert_arraysequal(fdata.inv_wgts[0][1], [0.5, 0.25])
-        sumsvd = fdata.correction
+        self.assert_arraysequal(yp_pdf.mean, [1., 10.])
+        self.assert_arraysequal(yp_pdf.i_invwgts[0][0], [0,1])
+        self.assert_arraysequal(yp_pdf.i_invwgts[0][1], [0.5, 0.25])
+        sumsvd = sum(yp_pdf.correction)
         self.assertEqual([sumsvd.mean, sumsvd.sdev], [0, 0])
-        test_logdet(fdata, prior, y)
+        test_logdet(yp_pdf, prior, y)
 
 
         # case 3 - prior and data correlated
         one = gv.gvar(1,1e-3)
         yo['y'][0] *= one
         po['p'] *= one
-        x, y, prior, fdata = lsqfit._unpack_data(
+        x, y, prior, yp_pdf = lsqfit._unpack_data(
             data=yo, prior=po, svdcut=0.0, eps=None, debug=True,
             uncorrelated_data=False, **noiseargs
             )
         self.assertEqual(x,False)
         self.assert_gvclose(y['y'],yo['y'])
         self.assert_gvclose(prior['p'],po['p'])
-        self.assert_arraysequal(fdata.mean, [1,10,2])
-        self.assert_arraysequal(fdata.inv_wgts[0][0], [1])
-        self.assert_arraysequal(fdata.inv_wgts[1][0], [0, 2])
-        fd_icov = make_mat(fdata.inv_wgts, 3)
+        self.assert_arraysequal(yp_pdf.mean, [1,10,2])
+        self.assert_arraysequal(yp_pdf.i_invwgts[0][0], [1])
+        self.assert_arraysequal(yp_pdf.i_invwgts[1][0], [0, 2])
+        fd_icov = make_mat(yp_pdf.i_invwgts, 3)
         icov = np.linalg.inv(gv.evalcov(y['y'].tolist()+[po['p']]))
         self.assert_arraysclose(fd_icov,icov)
         icov = np.linalg.inv(gv.evalcov(np.concatenate((y.flat,prior.flat))))
         self.assert_arraysclose(fd_icov,icov)
-        test_logdet(fdata, prior, y)
+        test_logdet(yp_pdf, prior, y)
 
         # case 4 - vector p, no correlation
         yo = dict(y=[gv.gvar(1, 2),gv.gvar(10,4)])
         pvo = lsqfit._unpack_gvars([gv.gvar(1,2),gv.gvar(1,4)])
-        x, y, prior, fdata = lsqfit._unpack_data(
+        x, y, prior, yp_pdf = lsqfit._unpack_data(
             data=yo, prior=pvo, svdcut=0, eps=None, debug=True,
             uncorrelated_data=False, **noiseargs
             )
         self.assertEqual(x,False)
         self.assert_gvclose(y['y'],yo['y'])
         self.assert_gvclose(prior,pvo)
-        self.assert_arraysequal(fdata.mean,[1, 10, 1, 1])
-        self.assert_arraysequal(fdata.inv_wgts[0][0], [0, 1, 2, 3])
-        self.assert_arraysequal(fdata.inv_wgts[0][1],[0.5, 0.25, 0.5, 0.25])
-        test_logdet(fdata, prior, y)
+        self.assert_arraysequal(yp_pdf.mean,[1, 10, 1, 1])
+        self.assert_arraysequal(yp_pdf.i_invwgts[0][0], [0, 1, 2, 3])
+        self.assert_arraysequal(yp_pdf.i_invwgts[0][1],[0.5, 0.25, 0.5, 0.25])
+        test_logdet(yp_pdf, prior, y)
 
         # case 5 - vector p, correlation
         yo = dict(y=[gv.gvar(1, 2),gv.gvar(10,4)])
         pvo = lsqfit._unpack_gvars([gv.gvar(1,2),gv.gvar(1,4)])
         yo['y'][0] *= one
         pvo[0] *= one
-        x, y, prior, fdata = lsqfit._unpack_data(
+        x, y, prior, yp_pdf = lsqfit._unpack_data(
             data=yo, prior=pvo, svdcut=0, eps=None, debug=True,
             uncorrelated_data=False, **noiseargs
             )
         self.assertEqual(x,False)
         self.assert_gvclose(y['y'],yo['y'])
         self.assert_gvclose(prior,pvo)
-        self.assert_arraysequal(fdata.inv_wgts[0][0], [1, 3])
-        self.assertEqual(numpy.ndim(fdata.inv_wgts[0][1]), 1)
-        self.assert_arraysequal(fdata.inv_wgts[1][0], [0, 2])
-        self.assertEqual(numpy.ndim(fdata.inv_wgts[1][1]), 2)
-        fd_icov = make_mat(fdata.inv_wgts, 4)
+        self.assert_arraysequal(yp_pdf.i_invwgts[0][0], [1, 3])
+        self.assertEqual(numpy.ndim(yp_pdf.i_invwgts[0][1]), 1)
+        self.assert_arraysequal(yp_pdf.i_invwgts[1][0], [0, 2])
+        self.assertEqual(numpy.ndim(yp_pdf.i_invwgts[1][1]), 2)
+        fd_icov = make_mat(yp_pdf.i_invwgts, 4)
         icov = np.linalg.inv(gv.evalcov(y['y'].tolist()+prior.tolist()))
         self.assert_arraysclose(fd_icov,icov)
         icov = np.linalg.inv(gv.evalcov(np.concatenate((y.flat,prior.flat))))
         self.assert_arraysclose(fd_icov,icov)
 
-        test_logdet(fdata, prior, y)
+        test_logdet(yp_pdf, prior, y)
 
         # case 6 - (x,y,ycov)
         yo = dict(y=[gv.gvar(1, 2),gv.gvar(10,4)])
@@ -1020,20 +1063,20 @@ class test_lsqfit(unittest.TestCase,ArrayTests):
         po = lsqfit._unpack_gvars(dict(p=gv.gvar(2, 4)))
         xo = 20
         svdcut = None
-        x, y, prior, fdata = lsqfit._unpack_data(
+        x, y, prior, yp_pdf = lsqfit._unpack_data(
             data=(xo,yo_mean,yo_cov), prior=po, svdcut=svdcut,eps=None,
             debug=True, uncorrelated_data=False, **noiseargs
             )
         self.assertEqual(x,xo)
         self.assert_gvclose(y,yo['y'])
         self.assert_gvclose(prior['p'],po['p'])
-        self.assert_arraysequal(fdata.mean,[1,10,2])
-        self.assertEqual(set(fdata.inv_wgts[0][0]), set([0, 1, 2]))
+        self.assert_arraysequal(yp_pdf.mean,[1,10,2])
+        self.assertEqual(set(yp_pdf.i_invwgts[0][0]), set([0, 1, 2]))
         self.assert_arraysequal(
-            fdata.inv_wgts[0][1][fdata.inv_wgts[0][0]],
+            yp_pdf.i_invwgts[0][1][yp_pdf.i_invwgts[0][0]],
             [0.5, 0.25, 0.25]
             )
-        test_logdet(fdata, prior, y)
+        test_logdet(yp_pdf, prior, y)
 
         # case 8 - svd cuts
         a = gv.gvar(1,1)
@@ -1045,7 +1088,7 @@ class test_lsqfit(unittest.TestCase,ArrayTests):
         sc = 0.01
         da_svd = gv.gvar(0,sc**0.5)
         for svdcut in [0.0, sc]:
-            x, y, prior, fdata = lsqfit._unpack_data(
+            x, y, prior, yp_pdf = lsqfit._unpack_data(
                 data=yo, prior=po, svdcut=svdcut, eps=None, debug=True,
                 uncorrelated_data=False, **noiseargs
                 )
@@ -1068,10 +1111,10 @@ class test_lsqfit(unittest.TestCase,ArrayTests):
                 with self.assertRaises(AssertionError):
                     self.assert_gvclose(prior,po)
             self.assert_arraysclose(
-                make_mat(fdata.inv_wgts, 4),
+                make_mat(yp_pdf.i_invwgts, 4),
                 np.linalg.inv(gv.evalcov(list(y.flat) + list(prior.flat)))
                 )
-            test_logdet(fdata, prior, y)
+            test_logdet(yp_pdf, prior, y)
 
         # case 9 - noise
         a = gv.gvar(1, 1)
@@ -1082,13 +1125,13 @@ class test_lsqfit(unittest.TestCase,ArrayTests):
         po = gv.gvar([20*(a+da), 20*(a-da), '10(10)'])
         sc = 0.01
         da_svd = gv.gvar(0, sc**0.5)
-        x, y_ref, prior_ref, fdata_ref = lsqfit._unpack_data(
+        x, y_ref, prior_ref, yp_pdf_ref = lsqfit._unpack_data(
             data=yo, prior=po, svdcut=sc, eps=None, debug=True,
             uncorrelated_data=False, **noiseargs
             )
         # 9a - check that it runs (all need do here)
         noiseargs['noise'] = (True, False)
-        x, y, prior, fdata = lsqfit._unpack_data(
+        x, y, prior, yp_pdf = lsqfit._unpack_data(
             data=yo, prior=po, svdcut=sc, eps=None, debug=True,
             uncorrelated_data=False, **noiseargs
             )
@@ -1096,12 +1139,12 @@ class test_lsqfit(unittest.TestCase,ArrayTests):
         # others: svdcut= #
         # case ?? - wrong length data tuple
         with self.assertRaises(ValueError):
-            x, y, prior, fdata = lsqfit._unpack_data(
+            x, y, prior, yp_pdf = lsqfit._unpack_data(
                 data=(xo,yo,yo,yo), prior=po, svdcut=svdcut, eps=None, debug=True,
                 uncorrelated_data=False, **noiseargs
                     )
         with self.assertRaises(ValueError):
-            x, y, prior, fdata = lsqfit._unpack_data(
+            x, y, prior, yp_pdf = lsqfit._unpack_data(
                 data=(xo,), prior=po, svdcut=svdcut, eps=None, debug=True,
                 uncorrelated_data=False, **noiseargs
                     )
@@ -1144,6 +1187,49 @@ class test_lsqfit(unittest.TestCase,ArrayTests):
         self.assertAlmostEqual(fit1.pmean['p'], fit2.pmean['p'], places=3)
         self.assertAlmostEqual(fit1.psdev['p'] * 2, fit2.psdev['p'], places=3)
 
+    def test_fake_fitargs(self):
+        """ fake_fitargs """
+        def f(x, p):
+            return p[0] + p[1] * x + p[2] * x ** 2
+        prior = gv.gvar(3 * ['0(1)'])
+        x = np.linspace(0.1, 1, 4)
+        fitargs = fake_fitargs(fcn=f, prior=prior, x=x)
+        fit = nonlinear_fit(**fitargs)
+        chi2 = gv.chi2(fit.p, fitargs['p0'])
+        self.assertLess(chi2 / chi2.dof, 6.)
+        self.assertLess(fit.chi2 / fit.dof, 6.)
+        # no correlations, noise
+        fitargs = fake_fitargs(fcn=f, prior=prior, x=x, correlate=False, noise=False)
+        fit = nonlinear_fit(**fitargs)
+        chi2 = gv.chi2(fit.p, fitargs['p0'])
+        self.assertLess(chi2 / chi2.dof, 6.)
+        self.assertLess(fit.chi2 / fit.dof, 6.)
+        self.assertAlmostEqual(np.sum(gv.evalcorr(fit.y)), len(x))
+        # no x
+        def g1(p, x=x):
+            return f(x, p)
+        fitargs = fake_fitargs(fcn=g1, prior=prior)
+        fit = nonlinear_fit(**fitargs)
+        chi2 = gv.chi2(fit.p, fitargs['p0'])
+        self.assertLess(chi2 / chi2.dof, 6.)
+        self.assertLess(fit.chi2 / fit.dof, 6.)
+        # fcn returns a dict
+        def g2(p, x=x):
+            return dict(ans=f(x,p))
+        fitargs = fake_fitargs(fcn=g2, prior=prior)
+        fit = nonlinear_fit(**fitargs)
+        chi2 = gv.chi2(fit.p, fitargs['p0'])
+        self.assertLess(chi2 / chi2.dof, 6.)
+        self.assertLess(fit.chi2 / fit.dof, 6.)
+        # p is a dict
+        def g3(p, x=x):
+            return f(x, p['c'])
+        fitargs = fake_fitargs(fcn=g3, prior=dict(c=prior))
+        fit = nonlinear_fit(**fitargs)
+        chi2 = gv.chi2(fit.p, fitargs['p0'])
+        self.assertLess(chi2 / chi2.dof, 6.)
+        self.assertLess(fit.chi2 / fit.dof, 6.)
+    
 
     def test_unpack_p0(self):
         """ _unpack_p0 """
@@ -1809,137 +1895,11 @@ class test_lsqfit(unittest.TestCase,ArrayTests):
         for a, x, gax, gxa in cases:
             np.testing.assert_allclose(gax, gammaQ(a, x), rtol=0.01)
             np.testing.assert_allclose(gxa, gammaQ(x, a), rtol=0.01)
+        from lsqfit._scipy import gammaQ as scipy_gammaQ
+        for a, x, gax, gxa in cases:
+            np.testing.assert_allclose(gax, scipy_gammaQ(a, x), rtol=0.01)
+            np.testing.assert_allclose(gxa, scipy_gammaQ(x, a), rtol=0.01)
 
-    @unittest.skipIf(
-        not hasattr(lsqfit, 'BayesIntegrator'),
-        "because no vegas module"
-        )
-    # @unittest.skipIf(FAST,"for speed")
-    def test_bayesintegrator_dict(self):
-        " BayesIntegrator(fit) "
-        neval = 250
-        # linear fit => BayesIntegrator gives same results for everything, norm=1
-        x = np.array([0.2, 0.6, 0.8, 1.2, 1.4])
-        y = gv.gvar(['0.38(20)', '0.85(20)', '0.59(20)', '1.44(20)', '0.73(20)'])
-        prior = gv.BufferDict(c=['0(5)', '0(5)'])
-        def f(x, p):
-            c = p['c']
-            return c[0] + c[1] * x
-        fit = nonlinear_fit(data=(x, y), prior=prior, fcn=f)
-
-        expval = BayesIntegrator(fit, limit=7.)
-        norm = expval(neval=neval, nitn=5).norm
-        self.assertTrue(abs(norm.mean - 1) <= 5. * norm.sdev)
-
-        def g(p):
-            c = p['c']
-            return dict(mean=c, outer=np.outer(c, c))
-        r = expval(g, neval=neval, nitn=5, adapt=False)
-        self.assertTrue(abs(r.norm.mean - 1) <= 5. * r.norm.sdev)
-        mean = r['mean']
-        cov = r['outer'] - np.outer(mean, mean)
-        dmean = mean - fit.pmean['c']
-        dcov = cov - gv.evalcov(fit.p['c'])
-        dnorm = r.norm - 1.
-        for dd in [dmean, dcov]:
-            self.assertTrue(
-                np.all(gv.mean(dd) ** 2 < 25. * gv.var(dd))
-                )
-
-        pdf = BayesPDF(fit)
-        r = expval(g, neval=neval, nitn=5, pdf=pdf, adapt=False)
-        mean = r['mean']
-        cov = r['outer'] - np.outer(mean, mean)
-        dmean = mean - fit.pmean['c']
-        dcov = cov - gv.evalcov(fit.p['c'])
-        dnorm = r.norm - 1.
-        for dd in [dmean, dcov]:
-            self.assertTrue(
-                np.all(gv.mean(dd) ** 2 < 25. * gv.var(dd))
-                )
-
-        def ga(p):
-            return p['c']
-        r = expval(ga, neval=neval, nitn=5, adapt=False)
-        self.assertTrue(abs(r.norm.mean - 1) <= 5. * r.norm.sdev)
-        dmean = r - fit.pmean['c']
-        self.assertTrue(
-            np.all(gv.mean(dmean) ** 2 < 25. * gv.var(dmean))
-            )
-
-        def gs(p):
-            return p['c'][0]
-        r = expval(gs, neval=neval, nitn=5, adapt=False)
-        self.assertTrue(abs(r.norm.mean - 1) <= 5. * r.norm.sdev)
-        dmean = r - fit.pmean['c'][0]
-        self.assertTrue(abs(dmean.mean) < 5. * dmean.sdev)
-
-    @unittest.skipIf(
-        not hasattr(lsqfit, 'BayesIntegrator'),
-        "because no vegas module"
-        )
-    @unittest.skipIf(FAST,"for speed")
-    def test_bayesintegrator_array(self):
-        " BayesIntegrator(fit) "
-        neval = 250
-        # linear fit => BayesIntegrator gives same results for everything, norm=1
-        x = np.array([0.2, 0.6, 0.8, 1.2, 1.4])
-        y = gv.gvar(['0.38(20)', '0.85(20)', '0.59(20)', '1.44(20)', '0.73(20)'])
-        prior = gv.gvar(['0(5)', '0(5)'])
-        def f(x, p):
-            return p[0] + p[1] * x
-        fit = nonlinear_fit(data=(x, y), prior=prior, fcn=f)
-
-        expval = BayesIntegrator(fit, limit=7.)
-        norm = expval(neval=neval, nitn=5).norm
-        self.assertTrue(abs(norm.mean - 1) <= 5. * norm.sdev)
-
-        def g(p):
-            return dict(mean=p, outer=np.outer(p, p))
-        r = expval(g, neval=neval, nitn=5, adapt=False)
-        self.assertTrue(abs(r.norm.mean - 1) <= 5. * r.norm.sdev)
-        mean = r['mean']
-        cov = r['outer'] - np.outer(mean, mean)
-        dmean = mean - fit.pmean
-        dcov = cov - gv.evalcov(fit.p)
-        dnorm = r.norm - 1.
-        for dd in [dmean, dcov]:
-            self.assertTrue(
-                np.all(gv.mean(dd) ** 2 < 25. * gv.var(dd))
-                )
-
-        pdf = BayesPDF(fit)
-        r = expval(g, neval=neval, nitn=5, pdf=pdf, adapt=False)
-        mean = r['mean']
-        cov = r['outer'] - np.outer(mean, mean)
-        dmean = mean - fit.pmean
-        dcov = cov - gv.evalcov(fit.p)
-        dnorm = r.norm - 1.
-        for dd in [dmean, dcov]:
-            self.assertTrue(
-                np.all(gv.mean(dd) ** 2 < 25. * gv.var(dd))
-                )
-
-        def ga(p):
-            return p
-        r = expval(ga, neval=neval, nitn=5, adapt=False)
-        self.assertTrue(abs(r.norm.mean - 1) <= 5. * r.norm.sdev)
-        dmean = r - fit.pmean
-        self.assertTrue(
-            np.all(gv.mean(dmean) ** 2 < 25. * gv.var(dmean))
-            )
-
-        def gs(p):
-            return p[0]
-        r = expval(gs, neval=neval, nitn=5, adapt=False)
-        self.assertTrue(abs(r.norm.mean - 1) <= 5. * r.norm.sdev)
-        dmean = r - fit.pmean[0]
-        self.assertTrue(abs(dmean.mean) < 5. * dmean.sdev)
-
-        r = expval(gs, neval=neval, nitn=5, adapt_to_pdf=False, adapt=False)
-        self.assertTrue(abs(r.norm.mean - 1) <= 5. * r.norm.sdev)
-        dmean = r - fit.pmean[0]
-        self.assertTrue(abs(dmean.mean) < 5. * dmean.sdev)
 
 def partialerrors(outputs,inputs):
     err = {}
