@@ -10,7 +10,7 @@ from lsqfit import MultiFitter, MultiFitterModel
 
 class test_multifitter(unittest.TestCase):
     def setUp(self):
-        gv.ranseed(1)
+        gv.ranseed(12345)
         # a = gv.gvar('1.000(1)')
         # b = gv.gvar('0.500(1)')
         a = gv.gvar('1.0(1)')
@@ -23,7 +23,7 @@ class test_multifitter(unittest.TestCase):
             ans['c2'] = 4 * [p['a']]
             return ans
         self.prior = gv.BufferDict([('a', a), ('b', b)])
-        self.data = gv.make_fake_data(fcn(self.prior))
+        self.data = gv.make_fake_data(fcn(self.prior))  # with gv._GVAR_RNG = np.random
         self.fcn = fcn
         # reference fit without using MultiFitter
         self.ref_fit = lsqfit.nonlinear_fit(
@@ -35,7 +35,7 @@ class test_multifitter(unittest.TestCase):
     def agree_ref(self, p):
         r = self.ref_fit.p 
         for k in ['a', 'b']:
-            if abs(r[k].mean - p[k].mean) > r[k].sdev:
+            if abs(r[k].mean - p[k].mean) > 5 * r[k].sdev:
                 return False 
             if abs(r[k].sdev - p[k].sdev) > r[k].sdev:
                 return False 
@@ -523,9 +523,16 @@ class test_multifitter(unittest.TestCase):
         def fitargs(z):
             prior = dict(a=gv.gvar(1.0, z), b=gv.gvar(0.5, z))
             return dict(prior=prior, data=self.data, chained=False)
-        fit,z = fitter.empbayes_fit(0.1, fitargs, step=0.01)
+        fit,z = fitter.empbayes_fit(0.1, fitargs, p0=dict(a=2.0, b=1.), step=0.01)
         self.assertNotAlmostEqual(z, 0.1)
         self.assertGreater(fit.logGBF, self.ref_fit.logGBF)
+        def fitargs2(z):
+            prior = dict(a=gv.gvar(1.0, z), b=gv.gvar(0.5, z))
+            return dict(prior=prior, data=self.data, chained=False, p0=dict(a=20., b=20.))
+        fit2,z2 = fitter.empbayes_fit(0.1, fitargs2, p0=dict(a=2.0, b=1.), step=0.01)
+        self.assertAlmostEqual(abs(z), abs(z2))
+        self.assertGreater(fit.logGBF, self.ref_fit.logGBF)
+        self.assertGreater(fit2.nit, fit.nit)
 
 class Linear(MultiFitterModel):
     def __init__(self, datatag, a, b, x, ncg):
